import * as THREE from 'three';
import { OrbitControls } from 'three/examples/jsm/controls/OrbitControls';

import { Loader } from './loader';

export class Editor {
    public loader: Loader;

    private scene: THREE.Scene;
    private grid: THREE.GridHelper;

    private viewportCamera: THREE.PerspectiveCamera;

    private renderer: THREE.WebGLRenderer;
    private orbitControl: OrbitControls;

    private viewportPanel: HTMLDivElement;

    constructor() {
        // this panel is normally part of more complex gui
        // and the style is normally in a more complex css file
        // i keep it just to be closer to my real structure
        this.viewportPanel = document.createElement( 'div' );
        this.viewportPanel.style.position = "absolute";
        this.viewportPanel.style.top = "0px";
        this.viewportPanel.style.left = "0px";
        this.viewportPanel.style.right = "0px";
        this.viewportPanel.style.bottom = "0px";
        this.viewportPanel.style.border = "none";
        document.body.appendChild( this.viewportPanel );
        this.viewportPanel.addEventListener( 'dragover', ( event: any ) => {
            event.preventDefault();
            event.dataTransfer.dropEffect = 'copy';
        }, false );
        this.viewportPanel.addEventListener( 'drop', this.dragDrop, false );

        // stipped version of the loader from the three editor
        this.loader = new Loader( this );

        // scene with a background
        this.scene = new THREE.Scene();
        this.scene.name = 'Scene';
        this.scene.background = new THREE.Color( 0x333333 );

        // simple grid
        this.grid = new THREE.GridHelper( 30, 30, 0xcccccc, 0x666666 );
        this.grid.name = 'Grid';

        // the camera
        this.viewportCamera = new THREE.PerspectiveCamera( 50, this.viewportPanel.offsetWidth / this.viewportPanel.offsetHeight, 0.01, 1000 );
        this.viewportCamera.name = 'Camera';
        this.viewportCamera.position.set( 0, 5, 10 );
        this.viewportCamera.lookAt( new THREE.Vector3() );

        // basic renderer
        this.renderer = new THREE.WebGLRenderer( { antialias:true } );
        this.renderer.setPixelRatio( window.devicePixelRatio );
        this.renderer.setSize( this.viewportPanel.offsetWidth, this.viewportPanel.offsetHeight );
        this.viewportPanel.appendChild( this.renderer.domElement );

        // basic lighting
        let ambient = new THREE.AmbientLight( 0x222222 );
        ambient.name = 'AmbientLight';
        this.scene.add( ambient );

        let directional: THREE.DirectionalLight = new THREE.DirectionalLight( 0xffffff, 1 );
        directional.name = 'DiectionalLight';
        directional.target.name = 'DirectionalLight Target';
        directional.position.set( 5, 10, 7.5 );
        this.scene.add( directional );
        let directionalHelper = new THREE.DirectionalLightHelper( directional, 1 );
        directionalHelper.name = 'DiectionalLight Helper';
        this.scene.add( directionalHelper );

        // and the orbit control
        this.orbitControl = new OrbitControls( this.viewportCamera, this.renderer.domElement );
        this.orbitControl.update();
    }

    private dragDrop = ( event: any ) => {
        event.preventDefault();
        if ( event.dataTransfer.types[ 0 ] === 'text/plain' ) return;
        if ( event.dataTransfer.items ) {
            this.loader.loadItemList( event.dataTransfer.items );
        } else {
            this.loader.loadFiles( event.dataTransfer.files );
        }
    }

    public loadObjectAndAnimation( object: any, animations?: any ): void {
        // in this stipped test there is no resources management at all
        // the object is simply straight added to the scene
        this.scene.add( object );
    }

    public onWindowResize(): void {
        this.viewportCamera.aspect = this.viewportPanel.offsetWidth / this.viewportPanel.offsetHeight;
        this.viewportCamera.updateProjectionMatrix();
        this.renderer.setSize( this.viewportPanel.offsetWidth, this.viewportPanel.offsetHeight );
    }

    public render(): void {
        this.scene.add( this.grid );
        this.renderer.render( this.scene, this.viewportCamera );
        this.scene.remove( this.grid );
    }
}